unit PromptEdit;
{ Copyright 2001 -  by Eddie Shipman                                           }
{ **************************************************************************** }
{ Hybrid TEdit that Combines Prompt Text and Edit like Intuit's Quicken and    }
{ Peachtree's Accounting for Windows.                                          }
{                                                                              }
{ Based upon the grey edit control seen at:                                    }
{ http://www.codeguru.com/editctrl/GreyEdit.shtml                              }
{                                                                              }
{ Inherited from TCustomMaskEdit due to future consideration for a data-aware  }
{ version.                                                                     }
{                                                                              }
{ This component is useful for instances when you do not have room for         }
{ corresponding label controls to set next to the TEdits. Another use is where }
{ you, as designer, want the form to look more like a paper form, such as      }
{ an invoice, purchaseorder, etc.                                              }
{                                                                              }
{ It lets you set the prompt for the data that goes in the field in the field  }
{ itself.                                                                      }
{                                                                              }
{ The user can tell between a prompt and real data using these rules:          }
{                                                                              }
{ 1. If the field has a value, display the value in the normal font.           }
{ 2. If the field has focus and is empty, then set the font to the             }
{    normal font and display a blank.                                          }
{ 3. If the field does not have focus and is empty, then show the Prompt       }
{    Text in the PromptFontColor and the PromptFontStyle.                      }
{                                                                              }
{ **************************************************************************** }
{ Added these properties:                                                      }
{ property Prompt: TCaption                                                    }
{   Set this as the Prompt for the field.                                      }
{                                                                              }
{ property PromptFontColor                                                     }
{   Set this to the color you want the Prompt Text to be.                      }
{     Defaults to clGray.                                                      }
{                                                                              }
{ property PromptFontStyles                                                    }
{   Set this to the FontStyle you want the Prompt Text to be.                  }
{   Defaults to [fsItalic].                                                    }
{                                                                              }
{ **************************************************************************** }

interface

uses
  Windows, Messages, Classes, SysUtils, Graphics, Controls, Mask;

type
  TCustomPromptEdit = class(TCustomMaskEdit)
  private
    { Private declarations }
    FNormalFontColor: TColor;
    FNormalFontStyles: TFontStyles;
    FOldFontColor: TColor;
    FOldFontStyles: TFontStyles;
    FOldText: TCaption;

    FPrompt: TCaption;
    FPromptFontColor: TColor;
    FPromptFontStyles: TFontStyles;
    procedure SetPrompt(Value: TCaption);
    procedure SetPromptFontColor(Value: TColor);
    function  GetPromptFontStyles: TFontStyles;
    procedure SetPromptFontStyles(Value: TFontStyles);
    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    { Protected declarations }
    procedure DoExit; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Prompt: TCaption read FPrompt write SetPrompt;
    property PromptFontColor: TColor read FPromptFontColor write SetPromptFontColor;
    property PromptFontStyles: TFontStyles read GetPromptFontStyles write SetPromptFontStyles;
  end;

  TPromptEdit = class(TCustomPromptEdit)
  published
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property Prompt;
    property PromptFontColor;
    property PromptFontStyles;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('Samples', [TPromptEdit]);
end;

procedure TCustomPromptEdit.SetPrompt(Value: TCaption);
begin
  if FPrompt <> Value then
  begin
    FPrompt := Value;
    Text := FPrompt;
    Invalidate;
  end;
end;

procedure TCustomPromptEdit.SetPromptFontColor(Value: TColor);
begin
  if FPromptFontColor <> Value then
  begin
    FPromptFontColor := Value;
    if Font.Color <> FPromptFontColor then
      Font.Color := FPromptFontColor;
    Invalidate;
  end;
end;

function  TCustomPromptEdit.GetPromptFontStyles: TFontStyles;
begin
  Result := FPromptFontStyles;
end;

procedure TCustomPromptEdit.SetPromptFontStyles(Value: TFontStyles);
begin
  if FPromptFontStyles <> Value then
  begin
    FPromptFontStyles := Value;
    Invalidate;
  end;
end;

procedure TCustomPromptEdit.CMExit(var Message: TCMExit);
begin
  DoExit;
end;

procedure TCustomPromptEdit.DoExit;
begin
  // If the Text hasn't changed, meaning that the user hasn't entered
  // anything, change the font and Text values back to what they were
  // when we entered {which should be clGray and Italic}
  if (Text = Prompt) or (Trim(Text) = '')then
  begin
    Font.Color := FOldFontColor;
    Font.Style := FOldFontStyles;
    Text := FOldtext;
  end
  else
    // Leave the Fonts the same color and style as they now are
    // but change the text back to what it was when we entered
    // the component.
    FOldText := FPrompt;
  if Assigned(OnExit) then OnExit(Self);
end;

procedure TCustomPromptEdit.CMEnter(var Message: TCMGotFocus);
begin
  if AutoSelect and not (csLButtonDown in ControlState) and
    (GetWindowLong(Handle, GWL_STYLE) and ES_MULTILINE = 0) then inherited SelectAll;
  // If the value has not yet been changed, save the current values
  // in the FOldText, FOldFontColor and FOldFontStyles variables.
  // Then set the Font properties for Normal Fonts and clear the
  // Text from the field.
  //
  // Basically, we want to clear the Text Property upon enter if the user hasn't
  // yet entered any data in the Text Property.

  if Text = FPrompt then
  begin
    FOldText := Text;
    FOldFontColor := Font.Color;
    FOldFontStyles := Font.Style;
    Font.Color := FNormalFontColor;
    Font.Style := FNormalFontStyles;
    inherited Clear;
  end;
  inherited;
end;

constructor TCustomPromptEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPromptFontColor := clGray;
  FPromptFontStyles:= [fsItalic];
  FNormalFontColor := clWindowText;
  FNormalFontStyles:= [];
  Font.Color       := FPromptFontColor;
  Font.Style       := FPromptFontStyles
end;


end.
